#!/usr/bin/env python2.7

import json
import logging
import os
import socket
import sqlite3
import sys

import tornado.ioloop
import tornado.options
import tornado.web

# Constants

PORT     = 9999
DB_PATH  = 'students.db'
TSV_PATH = 'students.tsv'

# Database

class Database(object):
    def __init__(self):
        self.conn = sqlite3.connect(DB_PATH)

        with self.conn:
            curs      = self.conn.cursor()
            statement = '''
            CREATE TABLE IF NOT EXISTS Students(
                netid           TEXT NOT NULL PRIMARY KEY,
                classification  TEXT NOT NULL,
                first_name      TEXT NOT NULL,
                last_name       TEXT NOT NULL
            )
            '''
            curs.execute(statement)

            for line in open(TSV_PATH):
                netid, classification, first_name, last_name = line.strip().split('\t')
                statement = '''
                INSERT OR REPLACE INTO Students (netid, classification, first_name, last_name) VALUES (?, ?, ?, ?)
                '''
                curs.execute(statement, (netid, classification, first_name, last_name))

    def search(self, field, query):
        with self.conn:
            curs      = self.conn.cursor()
            statement = 'SELECT * FROM Students WHERE {} LIKE ?'.format(field)
            return curs.execute(statement, ('%{}%'.format(query),))

# Handlers

class IndexHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('students.html')

class SearchHandler(tornado.web.RequestHandler):
    def get(self):
        # TODO: Get field and query value from form
        field    = self.get_argument('field', None)
        query    = self.get_argument('query', None)
        students = []

        # TODO: Search for students if field and query are valid
        if field and query:
            students = list(self.application.database.search(field, query))

        self.write(json.dumps({'results': students}))

# Application

class Application(tornado.web.Application):
    def __init__(self, port=PORT):
        tornado.web.Application.__init__(self, template_path='assets/html', debug=True)
        self.logger   = logging.getLogger()
        self.ioloop   = tornado.ioloop.IOLoop.instance()
        # TODO: Define database and port instance variables
        self.database = Database()
        self.port     = port

        # TODO: Add IndexHandler
        self.add_handlers('', [
            (r'/'           , IndexHandler),
            (r'/search'     , SearchHandler),
            (r'/assets/(.*)', tornado.web.StaticFileHandler, {'path': 'assets'}),
        ])

    def run(self):
        try:
            self.listen(self.port)
        except socket.error as e:
            self.logger.fatal('Unable to listen on {} = {}'.format(self.port, e))
            sys.exit(1)

        self.ioloop.start()

# Main Execution

if __name__ == '__main__':
    tornado.options.parse_command_line()

    application = Application()
    application.run()

# vim: set sts=4 sw=4 ts=8 expandtab ft=python:
